@@ -376,7 +376,7 @@ matches_text_list (GSList *text_list,
 		ECalComponentText *text;
 
 		text = l->data;
-		g_assert (text->value != NULL);
+		g_return_val_if_fail (text->value != NULL, FALSE);
 
 		if (e_util_utf8_strstrcasedecomp (text->value, str) != NULL) {
 			matches = TRUE;
