@@ -61,6 +61,7 @@ typedef struct {
 	gpointer get_message_data;
 	CamelMimeMessage *message;
 	CamelMessageInfo *info;
+	CamelFolder *folder;
 	const gchar *source;
 	GError **error;
 } FilterMessageSearch;
@@ -89,6 +90,7 @@ static CamelSExpResult *header_source (struct _CamelSExp *f, gint argc, struct _
 static CamelSExpResult *get_size (struct _CamelSExp *f, gint argc, struct _CamelSExpResult **argv, FilterMessageSearch *fms);
 static CamelSExpResult *pipe_message (struct _CamelSExp *f, gint argc, struct _CamelSExpResult **argv, FilterMessageSearch *fms);
 static CamelSExpResult *junk_test (struct _CamelSExp *f, gint argc, struct _CamelSExpResult **argv, FilterMessageSearch *fms);
+static CamelSExpResult *message_location (struct _CamelSExp *f, gint argc, struct _CamelSExpResult **argv, FilterMessageSearch *fms);
 
 /* builtin functions */
 static struct {
@@ -120,6 +122,7 @@ static struct {
 	{ "get-size",           (CamelSExpFunc) get_size,           0 },
 	{ "pipe-message",       (CamelSExpFunc) pipe_message,       0 },
 	{ "junk-test",          (CamelSExpFunc) junk_test,          0 },
+	{ "message-location",   (CamelSExpFunc) message_location,   0 }
 };
 
 static CamelMimeMessage *
@@ -873,20 +876,17 @@ junk_test (struct _CamelSExp *f,
 	gboolean message_is_junk = FALSE;
 	GError *error = NULL;
 
-	junk_filter = camel_session_get_junk_filter (fms->session);
-	if (junk_filter == NULL)
-		goto exit;
-
 	/* Check if the message is already classified. */
 
 	flags = camel_message_info_flags (info);
 
 	if (flags & CAMEL_MESSAGE_JUNK) {
+		message_is_junk = TRUE;
 		if (camel_debug ("junk"))
 			printf (
 				"Message has a Junk flag set already, "
 				"skipping junk test...\n");
-		goto exit;
+		goto done;
 	}
 
 	if (flags & CAMEL_MESSAGE_NOTJUNK) {
@@ -894,9 +894,22 @@ junk_test (struct _CamelSExp *f,
 			printf (
 				"Message has a NotJunk flag set already, "
 				"skipping junk test...\n");
-		goto exit;
+		goto done;
 	}
 
+	/* If the sender is known, the message is not junk.
+	   Do this before header test, to be able to override server-side set headers. */
+
+	sender_is_known = camel_session_lookup_addressbook (
+		fms->session, camel_message_info_from (info));
+	if (camel_debug ("junk"))
+		printf (
+			"Sender '%s' in book? %d\n",
+			camel_message_info_from (info),
+			sender_is_known);
+	if (sender_is_known)
+		goto done;
+
 	/* Check the headers for a junk designation. */
 
 	ht = camel_session_get_junk_headers (fms->session);
@@ -924,21 +937,41 @@ junk_test (struct _CamelSExp *f,
 		node = node->next;
 	}
 
-	/* If the sender is known, the message is not junk. */
+	/* Not every message info has headers available, thus try headers of the message itself */
+	message = camel_filter_search_get_message (fms, f);
+	if (message) {
+		struct _camel_header_raw *h;
 
-	sender_is_known = camel_session_lookup_addressbook (
-		fms->session, camel_message_info_from (info));
-	if (camel_debug ("junk"))
-		printf (
-			"Sender '%s' in book? %d\n",
-			camel_message_info_from (info),
-			sender_is_known);
-	if (sender_is_known)
+		for (h = CAMEL_MIME_PART (message)->headers; h; h = h->next) {
+			const gchar *value;
+
+			if (!h->name)
+				continue;
+
+			value = g_hash_table_lookup ((GHashTable *) ht, h->name);
+			if (!value)
+				continue;
+
+			message_is_junk = camel_strstrcase (h->value, value) != NULL;
+
+			if (message_is_junk) {
+				if (camel_debug ("junk"))
+					printf (
+						"Message contains \"%s: %s\"",
+						h->name, value);
+				goto done;
+			}
+		}
+	} else {
 		goto done;
+	}
 
 	/* Consult 3rd party junk filtering software. */
 
-	message = camel_filter_search_get_message (fms, f);
+	junk_filter = camel_session_get_junk_filter (fms->session);
+	if (junk_filter == NULL)
+		goto done;
+
 	status = camel_junk_filter_classify (
 		junk_filter, message, NULL, &error);
 
@@ -966,7 +999,7 @@ junk_test (struct _CamelSExp *f,
 		}
 
 		if (camel_debug ("junk"))
-			g_print (
+			printf (
 				"Junk filter classification: %s\n",
 				status_desc);
 	} else {
@@ -976,19 +1009,80 @@ junk_test (struct _CamelSExp *f,
 		message_is_junk = FALSE;
 	}
 
-done:
+ done:
 	if (camel_debug ("junk"))
 		printf (
 			"Message is determined to be %s\n",
 			message_is_junk ? "*JUNK*" : "clean");
 
-exit:
 	r = camel_sexp_result_new (f, CAMEL_SEXP_RES_BOOL);
 	r->value.number = message_is_junk;
 
 	return r;
 }
 
+/* this is copied from Evolution's libemail-engine/e-mail-folder-utils.c */
+static gchar *
+mail_folder_uri_build (CamelStore *store,
+                       const gchar *folder_name)
+{
+	const gchar *uid;
+	gchar *encoded_name;
+	gchar *encoded_uid;
+	gchar *uri;
+
+	g_return_val_if_fail (CAMEL_IS_STORE (store), NULL);
+	g_return_val_if_fail (folder_name != NULL, NULL);
+
+	/* Skip the leading slash, if present. */
+	if (*folder_name == '/')
+		folder_name++;
+
+	uid = camel_service_get_uid (CAMEL_SERVICE (store));
+
+	encoded_uid = camel_url_encode (uid, ":;@/");
+	encoded_name = camel_url_encode (folder_name, "#");
+
+	uri = g_strdup_printf ("folder://%s/%s", encoded_uid, encoded_name);
+
+	g_free (encoded_uid);
+	g_free (encoded_name);
+
+	return uri;
+}
+
+static CamelSExpResult *
+message_location (struct _CamelSExp *f,
+		  gint argc,
+		  struct _CamelSExpResult **argv,
+		  FilterMessageSearch *fms)
+{
+	CamelSExpResult *r;
+	gboolean same = FALSE;
+
+	if (argc != 1 || argv[0]->type != CAMEL_SEXP_RES_STRING)
+		camel_sexp_fatal_error (f, _("Invalid arguments to (message-location)"));
+
+	if (fms->folder && argv[0]->value.string) {
+		CamelStore *store;
+		const gchar *name;
+		gchar *uri;
+
+		store = camel_folder_get_parent_store (fms->folder);
+		name = camel_folder_get_full_name (fms->folder);
+		uri = mail_folder_uri_build (store, name);
+
+		same = g_str_equal (uri, argv[0]->value.string);
+
+		g_free (uri);
+	}
+
+	r = camel_sexp_result_new (f, CAMEL_SEXP_RES_BOOL);
+	r->value.boolean = same;
+
+	return r;
+}
+
 /**
  * camel_filter_search_match:
  * @session:
@@ -996,6 +1090,7 @@ exit:
  * @data: data for above
  * @info:
  * @source:
+ * @folder: in which folder the message is stored
  * @expression:
  * @error: return location for a #GError, or %NULL
  *
@@ -1008,6 +1103,7 @@ camel_filter_search_match (CamelSession *session,
                            gpointer data,
                            CamelMessageInfo *info,
                            const gchar *source,
+			   CamelFolder *folder,
                            const gchar *expression,
                            GError **error)
 {
@@ -1015,6 +1111,7 @@ camel_filter_search_match (CamelSession *session,
 	CamelSExp *sexp;
 	CamelSExpResult *result;
 	gboolean retval;
+	GError *local_error = NULL;
 	gint i;
 
 	fms.session = session;
@@ -1023,7 +1120,8 @@ camel_filter_search_match (CamelSession *session,
 	fms.message = NULL;
 	fms.info = info;
 	fms.source = source;
-	fms.error = error;
+	fms.folder = folder;
+	fms.error = &local_error;
 
 	sexp = camel_sexp_new ();
 
@@ -1036,21 +1134,29 @@ camel_filter_search_match (CamelSession *session,
 
 	camel_sexp_input_text (sexp, expression, strlen (expression));
 	if (camel_sexp_parse (sexp) == -1) {
-		/* A filter search is a search through your filters,
-		 * ie. your filters is the corpus being searched thru. */
-		g_set_error (
-			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-			_("Error executing filter search: %s: %s"),
-			camel_sexp_error (sexp), expression);
+		if (!local_error) {
+			/* A filter search is a search through your filters,
+			 * ie. your filters is the corpus being searched thru. */
+			g_set_error (
+				&local_error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Error executing filter search: %s: %s"),
+				camel_sexp_error (sexp), expression);
+		}
 		goto error;
 	}
 
 	result = camel_sexp_eval (sexp);
 	if (result == NULL) {
-		g_set_error (
-			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-			_("Error executing filter search: %s: %s"),
-			camel_sexp_error (sexp), expression);
+		if (!local_error)
+			g_set_error (
+				&local_error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Error executing filter search: %s: %s"),
+				camel_sexp_error (sexp), expression);
+		goto error;
+	}
+
+	if (local_error) {
+		camel_sexp_result_free (sexp, result);
 		goto error;
 	}
 
@@ -1073,5 +1179,8 @@ camel_filter_search_match (CamelSession *session,
 
 	g_object_unref (sexp);
 
+	if (local_error)
+		g_propagate_error (error, local_error);
+
 	return CAMEL_SEARCH_ERROR;
 }
