@@ -905,12 +905,12 @@ add_special_info (CamelStore *store,
 		g_free (vinfo->full_name);
 		g_free (vinfo->display_name);
 	} else {
+		g_return_if_fail (parent != NULL);
+
 		/* There wasn't a Trash/Junk folder so create a new
 		 * folder entry. */
 		vinfo = camel_folder_info_new ();
 
-		g_assert (parent != NULL);
-
 		vinfo->flags |=
 			CAMEL_FOLDER_NOINFERIORS |
 			CAMEL_FOLDER_SUBSCRIBED;
@@ -2917,7 +2917,7 @@ camel_store_synchronize_finish (CamelStore *store,
  *
  * Returns: Whether succeeded.
  *
- * Since: 3.14
+ * Since: 3.16
  **/
 gboolean
 camel_store_maybe_run_db_maintenance (CamelStore *store,
