@@ -845,14 +845,12 @@ imapx_parse_ext_optional (CamelIMAPXInputStream *stream,
 			dinfo = g_malloc0 (sizeof (*dinfo));
 			dinfo->refcount = 1;
 			/* should be string */
-			tok = camel_imapx_input_stream_astring (
-				stream, &token, cancellable, NULL);
-
-			if (tok != IMAPX_TOK_STRING) {
-				g_set_error (
-					&local_error,
-					CAMEL_IMAPX_ERROR, 1,
-					"expecting string");
+			if (!camel_imapx_input_stream_astring (stream, &token, cancellable, &local_error)) {
+				if (!local_error)
+					g_set_error (
+						&local_error,
+						CAMEL_IMAPX_ERROR, 1,
+						"expecting string");
 				goto done;
 			}
 
@@ -1057,14 +1055,20 @@ imapx_parse_address_list (CamelIMAPXInputStream *stream,
 			addr = camel_header_address_new ();
 			addr->type = CAMEL_HEADER_ADDRESS_NAME;
 			camel_imapx_input_stream_nstring (stream, &token, cancellable, &local_error);
-			if (local_error)
+			if (local_error) {
+				camel_header_address_unref (addr);
 				goto error;
+			}
 
 			addr->name = g_strdup ((gchar *) token);
 			/* we ignore the route, nobody uses it in the real world */
 			camel_imapx_input_stream_nstring (stream, &token, cancellable, &local_error);
-			if (local_error)
+			if (local_error) {
+				camel_header_address_unref (addr);
 				goto error;
+			}
+
+			mbox = NULL;
 
 			/* [RFC-822] group syntax is indicated by a special
 			 * form of address structure in which the host name
@@ -1075,18 +1079,23 @@ imapx_parse_address_list (CamelIMAPXInputStream *stream,
 			 * mailbox name field holds the group name phrase. */
 
 			camel_imapx_input_stream_nstring (stream, (guchar **) &mbox, cancellable, &local_error);
-			if (local_error)
+			if (local_error) {
+				camel_header_address_unref (addr);
 				goto error;
+			}
 
 			mbox = g_strdup (mbox);
 
 			camel_imapx_input_stream_nstring (stream, &host, cancellable, &local_error);
-			if (local_error)
+			if (local_error) {
+				camel_header_address_unref (addr);
 				goto error;
+			}
 
 			if (host == NULL) {
 				if (mbox == NULL) {
 					group = NULL;
+					camel_header_address_unref (addr);
 				} else {
 					g_free (addr->name);
 					addr->name = mbox;
@@ -1337,10 +1346,7 @@ imapx_parse_body (CamelIMAPXInputStream *stream,
 				stream, tok, token, len);
 		} while (tok == '(');
 
-		camel_imapx_input_stream_astring (
-			stream, &token, cancellable, &local_error);
-
-		if (local_error)
+		if (!camel_imapx_input_stream_astring (stream, &token, cancellable, &local_error) || local_error)
 			goto error;
 
 		cinfo->type = camel_content_type_new (
@@ -2095,7 +2101,7 @@ imapx_parse_uids (CamelIMAPXInputStream *stream,
                   GError **error)
 {
 	GArray *array;
-	guchar *token;
+	guchar *token = NULL;
 	gchar **splits;
 	guint len, str_len;
 	gint tok, ii;
@@ -2107,6 +2113,11 @@ imapx_parse_uids (CamelIMAPXInputStream *stream,
 	if (tok < 0)
 		return NULL;
 
+	if (!token) {
+		g_set_error (error, CAMEL_IMAPX_ERROR, 2, "server response truncated");
+		return NULL;
+	}
+
 	array = g_array_new (FALSE, FALSE, sizeof (guint32));
 	splits = g_strsplit ((gchar *) token, ",", -1);
 	str_len = g_strv_length (splits);
@@ -2182,6 +2193,7 @@ imapx_parse_status_copyuid (CamelIMAPXInputStream *stream,
 	GArray *uids;
 	guint64 number;
 	gboolean success;
+	GError *local_error = NULL;
 
 	success = camel_imapx_input_stream_number (
 		stream, &number, cancellable, error);
@@ -2191,15 +2203,37 @@ imapx_parse_status_copyuid (CamelIMAPXInputStream *stream,
 
 	sinfo->u.copyuid.uidvalidity = number;
 
-	uids = imapx_parse_uids (stream, cancellable, error);
-	if (uids == NULL)
+	uids = imapx_parse_uids (stream, cancellable, &local_error);
+	if (uids == NULL) {
+		/* Some broken servers can return truncated response, like:
+		   B00083 OK [COPYUID 4154  ] COPY completed.
+		   Just ignore such server error.
+		*/
+		if (g_error_matches (local_error, CAMEL_IMAPX_ERROR, 2)) {
+			g_clear_error (&local_error);
+			return TRUE;
+		}
+
+		if (local_error)
+			g_propagate_error (error, local_error);
+
 		return FALSE;
+	}
 
 	sinfo->u.copyuid.uids = uids;
 
 	uids = imapx_parse_uids (stream, cancellable, error);
-	if (uids == NULL)
+	if (uids == NULL) {
+		if (g_error_matches (local_error, CAMEL_IMAPX_ERROR, 2)) {
+			g_clear_error (&local_error);
+			return TRUE;
+		}
+
+		if (local_error)
+			g_propagate_error (error, local_error);
+
 		return FALSE;
+	}
 
 	sinfo->u.copyuid.copied_uids = uids;
 
