@@ -110,6 +110,13 @@ backend_network_monitor_can_reach_cb (GObject *source_object,
 
 	e_backend_set_online (backend, host_is_reachable);
 
+	if (!host_is_reachable) {
+		ESource *source;
+
+		source = e_backend_get_source (backend);
+		e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_DISCONNECTED);
+	}
+
 	g_object_unref (backend);
 }
 
@@ -560,6 +567,21 @@ backend_constructed (GObject *object)
 	}
 }
 
+static ESourceAuthenticationResult
+backend_authenticate_sync (EBackend *backend,
+			   const ENamedParameters *credentials,
+			   gchar **out_certificate_pem,
+			   GTlsCertificateFlags *out_certificate_errors,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	/* The default implementation just reports success, it's for backends
+	   which do not use (nor define) authentication routines, because
+	   they use different methods to get to the credentials. */
+
+	return E_SOURCE_AUTHENTICATION_ACCEPTED;
+}
+
 static gboolean
 backend_get_destination_address (EBackend *backend,
                                  gchar **host,
@@ -610,6 +632,7 @@ e_backend_class_init (EBackendClass *class)
 	object_class->finalize = backend_finalize;
 	object_class->constructed = backend_constructed;
 
+	class->authenticate_sync = backend_authenticate_sync;
 	class->get_destination_address = backend_get_destination_address;
 	class->prepare_shutdown = backend_prepare_shutdown;
 
@@ -1129,6 +1152,31 @@ e_backend_schedule_authenticate	(EBackend *backend,
 	g_clear_object (&cancellable);
 }
 
+/**
+ * e_backend_ensure_source_status_connected:
+ * @backend: an #EBackend
+ *
+ * Makes sure that the associated ESource::connection-status is connected. This is
+ * useful in cases when the backend can connect to the destination without invoking
+ * EBackend::authenticate_sync, possibly through e_backend_schedule_authenticate().
+ *
+ * Since: 3.16.4
+ **/
+void
+e_backend_ensure_source_status_connected (EBackend *backend)
+{
+	ESource *source;
+
+	g_return_if_fail (E_IS_BACKEND (backend));
+
+	source = e_backend_get_source (backend);
+
+	g_return_if_fail (E_IS_SOURCE (source));
+
+	if (e_source_get_connection_status (source) != E_SOURCE_CONNECTION_STATUS_CONNECTED)
+		e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_CONNECTED);
+}
+
 /**
  * e_backend_get_user_prompter:
  * @backend: an #EBackend
