@@ -1,17 +1,17 @@
 /*
  * module-cache-reaper.c
  *
- * This library is free software you can redistribute it and/or modify it
+ * This library is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  * for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -282,6 +282,9 @@ cache_reaper_scan_directory (ECacheReaper *extension,
 		if (g_strcmp0 (name, "system") == 0)
 			goto next;
 
+		if (g_strcmp0 (name, "rss") == 0)
+			goto next;
+
 		source = e_source_registry_server_ref_source (server, name);
 
 		if (source == NULL) {
